﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Kooboo.HtmlFunc.Function;
using Kooboo.HtmlFunc.Providers;

namespace Kooboo.HtmlFunc.Analysis.Visitors
{
    public class HtmlBlockPartVisitor : StartEndVisitor
    {
        public HtmlBlockPartVisitor(VisitorContext context)
            : base(context, FuncType.HtmlBlockPart)
        {
        }

        private IFuncTester _startCommentTester;
        protected override IFuncTester StartCommentTester
        {
            get
            {
                if (_startCommentTester == null)
                    _startCommentTester = new Testers.ParametricTester(this.Func.StartComment);
                return _startCommentTester;
            }
        }

        protected override object OnQualified(HtmlCommentNode startNode, HtmlCommentNode endNode)
        {
            var tester = (Testers.ParametricTester)this.StartCommentTester;
            var blockName = tester.GetParameter("name");
            if (string.IsNullOrEmpty(blockName))
            {
                if (!this.Context.ThrowException)
                    return null;
                throw new Exceptions.ParameterEmptyException(this.Context.FileName, startNode, "name");
            }

            // owner layout position
            int? order = null;
            IPosition ownerPosition = null;
            if (this.Context.Items.ContainsKey(FuncType.Position) && !this.Context.Items.ContainsKey(FuncType.ViewPart))
            {
                ownerPosition = (IPosition)this.Context.Items[FuncType.Position];
                order = ownerPosition.ChildCount++;
            }

            var block = Provider.Factory.GetProvider<IHtmlBlockPart>();
            block.BlockName = blockName;
            block.StartNode = startNode;
            block.EndNode = endNode;
            block.Order = order;
            block.OwnerPosition = ownerPosition;
            this.Context.Result.HtmlBlocks.Add(block);

            // ret context mark
            return block;
        }

        protected override FuncType ExclusiveType
        {
            get
            {
                return this.FuncType
                    | FuncType.Label
                    | FuncType.HtmlPart;//| FuncType.ViewPart;  // HtmlBlockPart can render by api directly.
            }
        }
    }
}
