﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HtmlAgilityPack;
using Kooboo.HtmlFunc.Function;
using Kooboo.HtmlFunc.Providers;

namespace Kooboo.HtmlFunc.Analysis.Visitors
{
    public class ViewPartVisitor : StartEndVisitor
    {
        public ViewPartVisitor(VisitorContext context)
            : base(context, FuncType.ViewPart)
        {
        }

        private IFuncTester _startCommentTester;
        protected override IFuncTester StartCommentTester
        {
            get
            {
                if (_startCommentTester == null)
                    _startCommentTester = new Testers.ParametricTester(this.Func.StartComment);
                return _startCommentTester;
            }
        }

        protected override object OnQualified(HtmlCommentNode startNode, HtmlCommentNode endNode)
        {
            var tester = (Testers.ParametricTester)this.StartCommentTester;
            var viewName = tester.GetParameter("name");
            if (string.IsNullOrEmpty(viewName))
            {
                if (!this.Context.ThrowException)
                    return null;
                throw new Exceptions.ParameterEmptyException(this.Context.FileName, startNode, "name");
            }

            // owner layout position
            int? order = null;
            IPosition ownerPosition = null;
            if (this.Context.Items.ContainsKey(FuncType.Position))
            {
                ownerPosition = (IPosition)this.Context.Items[FuncType.Position];
                order = ownerPosition.ChildCount++;
            }

            // add
            var viewPart = Provider.Factory.GetProvider<IViewPart>();
            viewPart.ViewName = viewName;
            viewPart.StartNode = startNode;
            viewPart.EndNode = endNode;
            viewPart.Order = order;
            viewPart.OwnerPosition = ownerPosition;
            this.Context.Result.ViewParts.Add(viewPart);

            // ret context mark
            return viewPart;
        }

        protected override FuncType ExclusiveType
        {
            get
            {
                return this.FuncType
                    | FuncType.Menu
                    | FuncType.Label
                    | FuncType.HtmlBlockPart
                    | FuncType.HtmlPart
                    | FuncType.List
                    | FuncType.Detail;
            }
        }
    }
}
